package net.wanji.opt.kafka;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.opt.common.Constants;
import net.wanji.opt.common.RedisUtils;
import net.wanji.opt.service.GreenBeltInfoService;
import net.wanji.opt.vo.GreenBeltInfoVO;
import org.apache.kafka.clients.consumer.Consumer;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author duanruiming
 * @date 2024/11/18 23:40
 * @description 消费神思推送kafka数据，不同kafka
 */
@Component
@Slf4j
public class KafkaListeners implements KafkaListenerErrorHandler {

    @Resource
    private GreenBeltInfoService greenBeltInfoService;
    @Resource
    private RedisUtils redisUtils;


    @KafkaListener(topics = {"green_belt_info_data"}, groupId = "group2", containerFactory = "kafkaListenerContainerFactory2")
    public void receiveGreenBeltInfoData(String message) throws Exception {
        if (StringUtils.isNotBlank(message)) {
            GreenBeltInfoVO greenBeltInfoVO = greenBeltInfoService.convertData(message);
            greenBeltInfoService.save(greenBeltInfoVO);
            String key = Constants.GREEN_ID_OPT_KEY.concat(greenBeltInfoVO.getGreenId());
            redisUtils.set(key, JacksonUtils.getInstance().writeValueAsString(greenBeltInfoVO));
        }
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }

}
